==> untahris-2.0-src.tgz: untahris/untahris.h <==
// Untahris (Common Playground)
// copyright (C) 2006 longuens@users.sourceforge.net
// Released under GNU General Public License (see the file COPYING)
// --------------------------------------------------------------
// Header file for everything.

#include <vector>
#include <string>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_mixer.h>

using namespace std;

#define MAXX 128
#define MAXY 128

#define NEVER 2000000000

int ky[4] = {-1, 0, 1, 0};
int kx[4] = {0, 1, 0, -1};

typedef char int8;
typedef short int16;
typedef long int32;
typedef long long int64;

enum PlayerKey {
  keyUp,
  keyRight,
  keyDown,
  keyLeft,
  keyFire,
  keyEarth,
  keyAir,
  keyWater,
  keyJoinGame // fake key
  };

enum CellType {
  cFree,
  cBreakWall,
  cPermaWall,

  cSnakeHead,
  cSnakeDead,
  cSnakeLeft,
  cSnakeRight,
  cSnakeUp,
  cSnakeDown,
  cSnakeTRON,
  
  cTetris,
  cTetrisDown,
  
  cBig,
  
  cCannonF,
  cCannonA,

  cBonusLevel,
  cBonusScore,
  cBonusAmmo,
  
  cCastleWall,

  cBomb1,
  cBomb2,
  cExplosion
  };

struct TBuffer;

enum IconId {
  icFree, icBreak, icPerma,
  icAmmo, icLevel, icScore, 
  icThrust, icBall, icBullet, icBomber, icBomb1, icBomb2, icSpawn,
  icExplosion, icExplosion1, icExplosion2, icExplosion3, 
  icCastle, icCannon, icCannonA, icCannonP, icTarget,
  icWorm, icBazooka,
  icLastIcon
  };

// display's icon indexed by given ID
#define Ic(x) icons[int(x)]
#define DIc(x) ow()->icons[int(x)]

static const int numIcons = int(icLastIcon);

static char* iconnames[numIcons] = {
  "FREE", "BREAK", "PERMA",
  "AMMO", "LEVEL", "SCORE",
  "THRUST", "BULLET", "BALL", "BOMBER", "BOMB1", "BOMB2", "SPAWN",
  "EXPLOSION", "EXPLOSION1", "EXPLOSION2", "EXPLOSION3", 
  "CASTLE", "CANNON", "CANNONA", "CANNONP", "TARGET",
  "WORM", "BAZOOKA"
  };

struct TIcon {
  int size;
  int pixel[64][64];
  };

struct TDisplayIcon {
  TIcon i;
  void loadFrom(FILE *f);
  void fillWith(char *chars, int size);
  void fillTile(TDisplayIcon &orig, int size);
  void gradFill(int x0, int y0, int x1, int y1, int c0, int c1);
  void gradBall(int x0, int y0, int x1, int y1, int c0, int c1);
  };

struct TPlayerIcon {
  TIcon i;
  int color;
  void drawAt(int x, int y);
  void assignColor(int c);
  void drawAtCentered(int x, int y) { drawAt(x-i.size/2, y-i.size/2); }
  void drawAtRotated(int x, int y, int rx, int ry);
  };

struct TPlayer {
  TPlayerIcon icons[numIcons];
  int8 index;
  int8 typenum;
  int16 lives, frags;
  int32 score, level, ammo, typeval;
  int16 spx, spy; // spawn location
  char *getTypeName();
  int32 color; // color used
  bool key[8]; // keys pressed
  void reactToKey(PlayerKey pk, bool pressed);
  void draw();
  void localDraw() {}
  void init(int c);
  bool useAmmo(int qty) 
    {if(ammo < qty) return false; ammo -= qty; return true;}
  void save(TBuffer& b);
  void load(TBuffer& b);
  void createIcons();
  };

struct TPlayObject {
  public:
  int8 owner;
  int32 time;
  int getTime() {return time;}
  virtual ~TPlayObject () {}
  virtual void reactToKey(PlayerKey pk, bool pressed) {}
  virtual void draw() {}
  virtual void go() {}
  TPlayer *ow();

  int spd() { return 7999 / (200 + ow()->level) + 1; }
  int scoreGain() { return (100+ow()->level)*(100+ow()->level)/1000; }
  int bulletSpeed() { return 50 + ow()->level/4; }
  virtual void save(TBuffer& b) = 0;
  
  // these methods are supposed to check whether the two collide
  virtual bool reactToBullet(struct OBullet* bullet) {return false;}
  // return true if the bullet is to disappear
  virtual bool reactToPad(struct OPad* pad) {return false;}
  // return true if it kills the pad (reactToPad destroys pad object)
  virtual bool reactToThruster(struct OThruster* bullet) {return false;}
  // return true if it kills the thruster (react destroy thuster)
  virtual bool smashedByBall(struct OBall* ball) {return false;}
  // return true if it changes the ball's direction
  virtual void destroyBalls() { }
  // react to 'no balls' command
  virtual bool catchesRope(int rx, int ry) {return false;}
  // does ninja rope at (rx,ry) hook to it?
  virtual void pullRope(int& rx, int& ry, int &drx, int &dry, int ddx, int ddy) {}
  // pull the ninja rope at (x,y)
  
  void killedBy(int player);
  // player killed me -- this deletes the object and scores a frag
  };

struct TPlayground {
  int32 time;
  int32 kqpos;
  
  int64 seed;
  int rand();

  int16 sizeX, sizeY;
  CellType map[MAXY][MAXX];
  char owner[MAXY][MAXX];
  
  void mapSet(int x, int y, CellType c, char o) {
    map[y][x] = c;
    owner[y][x] = o;
    }    
    
  vector<TPlayer*> pl;
  vector<TPlayObject*> objects;

  void upTime(int newTime);
  
  void cleanup();
  void init(int sx, int sy); 
  void generate();
  
  int addPlayer(TPlayer* p);
  
  void deleteObject(TPlayObject *obj);

  void save(TBuffer& b);
  void load(TBuffer& b);
  };

struct TKeypress {
  int32 time;
  bool pressed;
  int32 player; // int32 because also used for color
  PlayerKey key;
  };

template<class T> static inline int size(vector<T> v) {return v.size();}

enum TGameType {gLocal, gHost, gClient};

enum TWorldType {
  wtEmpty,
  wtCavern,
  wtSwissCheese,
  wtConstruction,
  wtFrame,
  wtGrid,
  wtRuins,
  wtWall,
  wtLastWorldType
  };

static const int numWorldTypes = int(wtLastWorldType);

static char* worldNames[numWorldTypes] = {
  "EMPTY", "CAVERN", "SWISS CHEESE", "CONSTRUCTION", "FRAME", "GRID", "RUINS", "WALL"
  };

struct TConfig {
  TGameType gt;
  string server;
  int  port;
  int  res;
  int  sizeX;
  int  sizeY;
  int  resX;
  int  resY;
  int  initialWorld;
  int  initBonus;

  bool fs;
  bool centered;

  string err;
  bool ok;
  };

struct TLocalPlayer {
  int index;   // index in the player table
  int keys[8]; // keys for this player
  int color;
  };

struct TDisplay {

  TDisplayIcon icons[numIcons];
  
  Mix_Chunk 
    *sndBirth, *sndDeath, 
    *sndBlue, *sndGreen, *sndRed,
    *sndShoot, *sndHit, *sndPlace;

  SDL_Surface *surface;
  TTF_Font *font;
  int sizeX;
  int sizeY;
  int ofsX;
  int ofsY;
  int Res;
  bool stop;
  
  int *bground; // background image
  
  int interpretPixel(char ch);
  
  int delta; // time difference
  
  int getCurrentTime();
  void setCurrentTime(int val);

  void init();
  
  void initForGame();
  
  void loop();
  
  void draw();
  
  void putpix(int x, int y, int c);
  
  void playSoundAt(Mix_Chunk *sound, int x, int y);
  
  void addKeypress(int key, bool type);
  
  void putText(int x, int y, int color, const char *text, int align);
  
  void menuLoop();
  void menuKey(int key);
  void menuDraw();
  
  int menupos;

  void genBGround(int x0, int y0, int x1, int y1, int sy);
  };

struct TException {
  string msg;
  int data;
  string ds;
  TException(string m, int d=0, string s="") {msg=m; data=d; ds=s;}
  };

#define KQSIZE 32768

TConfig c;
TDisplay d;
TPlayground g;
vector<TLocalPlayer> lps;

TKeypress kqueue[KQSIZE];
int kqend;

void addKeyToQueue(TKeypress& kp);

void startGame();
void dataTransfer();
void broadcastKey(TKeypress& kp);

==> untahris-3.1-src.tgz: untahris/untahris.h <==
// Untahris (Common Playground)
// copyright (C) 2006 longuens@users.sourceforge.net
// Released under GNU General Public License (see the file COPYING)
// --------------------------------------------------------------
// Header file for everything.

#include <vector>
#include <string>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_mixer.h>

using namespace std;

#define MAXX 128
#define MAXY 128

#define NEVER 2000000000

int ky[4] = {-1, 0, 1, 0};
int kx[4] = {0, 1, 0, -1};

typedef char int8;
typedef short int16;
typedef long int32;
typedef long long int64;

enum PlayerKey {
  keyUp,
  keyRight,
  keyDown,
  keyLeft,
  keyFire,
  keyEarth,
  keyAir,
  keyWater,
  // fake keys
  keyLMB,
  keyRMB,
  keyMoveMouse,
  keyJoinGame
  };

char* ctnames = ".#Xabcdefghi0FGLSRjklmxw";

enum CellType {
  cFree,
  cBreakWall,
  cPermaWall,

  cSnakeHead,
  cSnakeDead,
  cSnakeLeft,
  cSnakeRight,
  cSnakeUp,
  cSnakeDown,
  cSnakeTRON,
  
  cTetris,
  cTetrisDown,
  
  cBig,
  
  cCannonF,
  cCannonA,

  cBonusLevel,
  cBonusScore,
  cBonusAmmo,
  
  cCastleWall,

  cBomb1,
  cBomb2,
  cExplosion,
  
  cXunux,
  cXunuxStable
  };

struct TBuffer;

enum IconId {
  icFree, icBreak, icPerma,
  icAmmo, icLevel, icScore, 
  icThrust, icBall, icBullet, icBomber, icBomb1, icBomb2, icSpawn,
  icExplosion, icExplosion1, icExplosion2, icExplosion3, 
  icCastle, icCannon, icCannonA, icCannonP, icTarget,
  icWorm, icBazooka, icXunux, icIncastle,
  icLastIcon
  };

// display's icon indexed by given ID
#define Ic(x) icons[int(x)]
#define DIc(x) ow()->icons[int(x)]

static const int numIcons = int(icLastIcon);

static char* iconnames[numIcons] = {
  "FREE", "BREAK", "PERMA",
  "AMMO", "LEVEL", "SCORE",
  "THRUST", "BULLET", "BALL", "BOMBER", "BOMB1", "BOMB2", "SPAWN",
  "EXPLOSION", "EXPLOSION1", "EXPLOSION2", "EXPLOSION3", 
  "CASTLE", "CANNON", "CANNONA", "CANNONP", "TARGET",
  "WORM", "BAZOOKA", "XUNUX", "INCASTLE"
  };

struct TIcon {
  int size;
  int pixel[64][64];
  };

struct TDisplayIcon {
  TIcon i;
  void loadFrom(FILE *f);
  void fillWith(const char *chars, int size);
  void fillTile(TDisplayIcon &orig, int size);
  void gradFill(int x0, int y0, int x1, int y1, int c0, int c1);
  void gradBall(int x0, int y0, int x1, int y1, int c0, int c1);
  void drawLines(int x0, int y0, int x1, int y1, int color, int type);
  };

struct TPlayerIcon {
  TIcon i;
  int color;
  void drawAt(int x, int y);
  void drawAtFlipped(int x, int y);
  void assignColor(int c);
  void drawAtCentered(int x, int y) { drawAt(x-i.size/2, y-i.size/2); }
  void drawAtCenteredFlipped(int x, int y) { drawAtFlipped(x-i.size/2, y-i.size/2); }
  void drawAtRotated(int x, int y, int rx, int ry);
  };

struct TKeypress {
  int32 time;
  int8 pressed, mx, my, key;
  int32 player; // int32 because also used for color
  };

template<class T> static inline int size(vector<T> v) {return v.size();}

enum TGameType {gLocal, gHost, gClient};

enum TWorldType {
  wtEmpty,
  wtCavern,
  wtSwissCheese,
  wtConstruction,
  wtFrame,
  wtGrid,
  wtRuins,
  wtWall,
  wtLoad,
  wtLastWorldType
  };

static const int numWorldTypes = int(wtLastWorldType);

static char* worldNames[numWorldTypes] = {
  "EMPTY", "CAVERN", "SWISS CHEESE", "CONSTRUCTION", "FRAME", "GRID", "RUINS", "WALL", "SAVED MAP"
  };

enum TBackground {
  bgBlack,
  bgPlasma,
  bgGradient,
  bgGrid,
  bgGray
  };

enum TGraphicsType {
  gtGradient,
  gtRetro,
  gtLines,
  gtASCII
  };

static char* bgNames[5] = {"BLACK", "PLASMA", "GRADIENT", "GRID", "GRAY"};

static char* gtNames[3] = {"GRADIENT", "RETRO", "LINES"};

struct TEntityConfig {
  int32 initSpeed;
  int32 speedBonus;
  int32 maxSpeed;
  int32 initAmmo;
  int32 bonusAmmo;
  int32 ammoPerShoot;
  };

struct TConfig {
  TGameType gt;
  string server;
  string mapname;
  int  port;
  int  res;
  int  pctX, pctY, scorelines;
  int  resX;
  int  resY;
  int  initialWorld;
  int  initBonus;
  
  int  background;
  int  graphicsType;

  bool fs;
  bool centered;

  string err;
  bool ok;  
  };

struct TGameConfig {
  TEntityConfig ec[9];
  
  int32 ballSpeed, bombInitRange, bombBonusRange, castleInitEnergy, castleRotSpeed, castleRegenSpeed, castlePowerBase,
    ammoPerCannon, utetrisMinLength, initFuel, bonusFuel, thrustAcc, fuelUseSpeed, wormWalkSpeed, wormFlySpeed,
    ninjaRopeLength, ninjaPullSpeed;
  
  bool
    ballsTakeBonuses,
    regenerateLevel, regenerateScore, regenerateAmmo, regenerateDestroyed,
    
    bulletsKillBalls, cannonsKillBalls, explosionsDestroyBalls, ballsDestroyXunux, ballsDestroyWalls,
    xunuxKillsSnake, xunuxKillsThruster, xunuxKillsXunux, xunuxAttacksXunux;
  
  bool regenerate(CellType c) {
    if(c == cBonusLevel) return regenerateLevel;
    if(c == cBonusScore) return regenerateScore;
    if(c == cBonusAmmo ) return regenerateAmmo;
    }
  };

TConfig c;
TGameConfig gc;

struct TLocalPlayer {
  int index;   // index in the player table
  int keys[8]; // keys for this player
  int color;
  };

struct TDisplay {

  TDisplayIcon icons[numIcons];
  
  Mix_Chunk 
    *sndBirth, *sndDeath, 
    *sndBlue, *sndGreen, *sndRed,
    *sndShoot, *sndHit, *sndPlace;

  SDL_Surface *surface;
  TTF_Font *font;
  int sizeX;
  int sizeY;
  int ofsX;
  int ofsY;
  int Res;
  bool stop;
  
  int *bground; // background image
  
  int interpretPixel(char ch);
  
  int delta; // time difference
  
  int getCurrentTime();
  void setCurrentTime(int val);

  void init();

  void setMode();
  
  void initForGame();
  
  void initBackground();
  
  void loop();
  
  void draw();
  
  void putpix(int x, int y, int c);
  
  void playSoundAt(Mix_Chunk *sound, int x, int y);
  
  void addKeypress(int key, bool type);
  void addMouseEvent(int x, int y, int type, int button);
  
  void putText(int x, int y, int color, const char *text, int align);
  
  void menuLoop();
  void menuKey(int key);
  void menuDraw();
  
  int menupos;

  void genBGround(int x0, int y0, int x1, int y1, int sy);
  };

struct TException {
  string msg;
  int data;
  string ds;
  TException(string m, int d=0, string s="") {msg=m; data=d; ds=s;}
  };

struct TPlayer {
  TPlayerIcon icons[numIcons];
  int8 index;
  int8 editing, charid, editid;
  int16 lives, frags;
  int32 score, level, ammo, typeval;
  int16 killer;   // who killed us - used by Xunux
  char *getTypeName();
  int32 color; // color used
  bool key[8]; // keys pressed
  void localDraw() {}
  void init(int c);
  bool useAmmo(int qty) 
    {if(ammo < qty) return false; ammo -= qty; return true;}
  TEntityConfig& ec() { return gc.ec[charid]; }
  bool useAmmo() { useAmmo(ec().ammoPerShoot); }
  void gainAmmo() { ammo += ec().bonusAmmo; }
  void save(TBuffer& b);
  void load(TBuffer& b);
  void createIcons();
  };

struct TPlayObject {
  public:
  int8 owner;
  int32 time;
  int getTime() {return time;}
  virtual ~TPlayObject () {}
  virtual void reactToKey(PlayerKey pk, bool pressed) {}
  virtual void reactToKey(TKeypress& k);
  virtual void draw() {}
  virtual void go() {}
  virtual void pausedGo();
  virtual void delay(int t) { time += t; } // remove for non-paused objects
  TPlayer *ow();

  struct TEntityConfig& ec() { return ow()->ec(); }
  int spd();
  int scoreGain() { return (10+ow()->level)*(10+ow()->level)/10; }
  int bulletSpeed() { return 50 + ow()->level/4; }
  virtual void save(TBuffer& b) = 0;
  
  // these methods are supposed to check whether the two collide
  virtual bool reactToBullet(struct OBullet* bullet) {return false;}
  // return true if the bullet is to disappear
  virtual bool reactToPad(struct OPad* pad) {return false;}
  // return true if it kills the pad (reactToPad destroys pad object)
  virtual bool reactToThruster(struct OThruster* bullet) {return false;}
  // return true if it kills the thruster (react destroy thuster)
  virtual bool smashedByBall(struct OBall* ball) {return false;}
  // return true if it changes the ball's direction
  virtual void reactToCannon(struct OCannonball* ball) {}
  // return true if it changes the ball's direction
  virtual void destroyBalls() { }
  // react to 'no balls' command
  virtual bool catchesRope(int rx, int ry) {return false;}
  // does ninja rope at (rx,ry) hook to it?
  virtual void pullRope(struct OWorm *worm, int ddx, int ddy) {}
  // pull the ninja rope at (x,y)
  
  void killedBy(int player);
  // player killed me -- this deletes the object and scores a frag
  };

struct TPlayground {
  int32 time;
  int32 kqpos;
  bool  paused;
  
  int64 seed;
  int rand();

  int16 sizeX, sizeY;
  CellType map[MAXY][MAXX];
  char owner[MAXY][MAXX];
  
  void mapSet(int x, int y, CellType c, char o) {
    map[y][x] = c;
    owner[y][x] = o;
    }    
    
  vector<TPlayer*> pl;
  vector<TPlayObject*> objects;

  void upTime(int newTime);
  
  void cleanup();
  void init(int sx, int sy); 
  void generate();
  void saveMap();
  void destroyBalls();
  
  int addPlayer(TPlayer* p);
  
  void deleteObject(TPlayObject *obj);

  void save(TBuffer& b);
  void load(TBuffer& b);  
  };

char *charnames[10] = {
  "SNAKE",
  "UTRON",
  "UTETRIS",
  "THRUSTER",
  "BOMBER",
  "CASTLE",
  "WORM",
  "PAD",
  "XUNUX",
  "Ball"
  };

char *editnames[13] = {
  "-neutral ball-",
  "-break wall-",
  "-perma wall-",
  "-clear-",
  "-cannon-",
  "-bonus ammo-",
  "-bonus level-",
  "-bonus other-",
  "generate [F7]",
  "no balls [F8]",
  "save map [F2]",
  "pause game [F4]",
  "quit game [ESC]"
  };

#define tnSNAKE    0
#define tnUTRON    1
#define tnUTETRIS  2
#define tnTHRUSTER 3
#define tnBOMBER   4
#define tnCASTLE   5
#define tnWORM     6
#define tnPAD      7
#define tnXUNUX    8

#define KQSIZE 32768

TDisplay d;
TPlayground g;
vector<TLocalPlayer> lps;

TKeypress kqueue[KQSIZE];
int kqend;

void addKeyToQueue(TKeypress& kp);

void startGame();
void dataTransfer();
void broadcastKey(TKeypress& kp);
